package cd.casic.ci.process.process.service.aflManager.impl;

import cd.casic.ci.process.dto.req.aflManager.AflManagerReq;
import cd.casic.ci.process.dto.resp.aflManager.AflCrashesInfoResp;
import cd.casic.ci.process.dto.resp.resource.ResourceFindResp;
import cd.casic.ci.process.engine.constant.PipelineVariableConstant;
import cd.casic.ci.process.engine.manager.RunContextManager;
import cd.casic.ci.process.engine.runContext.BaseRunContext;
import cd.casic.ci.process.process.converter.AflManagerConverter;
import cd.casic.ci.process.process.dao.aflManager.AflCrashesInfoDao;
import cd.casic.ci.process.process.dataObject.aflManager.AflCrashesInfo;
import cd.casic.ci.process.process.dataObject.resource.PipResourceMachine;
import cd.casic.ci.process.process.service.aflManager.AflCrashesInfoService;
import cd.casic.ci.process.process.service.resource.ResourceManagerService;
import cd.casic.ci.process.util.CryptogramUtil;
import cd.casic.ci.process.util.SftpUploadUtil;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import de.danielbechler.util.Collections;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName TargetVersionServiceImpl
 * @Date: 2025/5/17 15:20
 * @Description:
 */
@Service
@Slf4j
public class AflCrashesInfoServiceImpl extends ServiceImpl<AflCrashesInfoDao, AflCrashesInfo> implements AflCrashesInfoService {

    private static final String remoteFilePath = "/home/casic/706/yunqi/";
    private static final String crashesFilePath = "/ai_afl/default/crashes/";

    @Resource
    private RunContextManager runContextManager;
    @Resource
    private ResourceManagerService resourceManagerService;



    @Override
    public String saveAflCrashesInfo(AflManagerReq req) {
        if (StringUtils.isEmpty(req.getPipelineId()) || StringUtils.isEmpty(req.getTaskId())){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"pipelineId或者taskId不能为空");
        }
        String uuid = UUID.randomUUID().toString();

        BaseRunContext runContext = runContextManager.getContext(req.getPipelineId());
        if (ObjectUtils.isEmpty(runContext)){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"未找到运行上下文");
        }
        String resourceId = runContext.getGlobalVariables().get(PipelineVariableConstant.AFL_RESOURCE_MANAGER_ID_KEY) instanceof String ? ((String) runContext.getGlobalVariables().get(PipelineVariableConstant.AFL_RESOURCE_MANAGER_ID_KEY)) : null;
        if (StringUtils.isEmpty(resourceId)){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"未找到资源");
        }

        ResourceFindResp resourceById = resourceManagerService.findResourceById(resourceId);
        if (resourceById == null || resourceById.getResourceMachine() == null) {
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"资源信息错误");
        }

        PipResourceMachine resourceMachine = resourceById.getResourceMachine();
        String password = CryptogramUtil.doDecrypt(resourceMachine.getPassword());
        resourceMachine.setPassword(password);

        try {
            // 步骤1：列出源目录下的所有文件
            List<String> files = SftpUploadUtil.listFilesInRemoteDirectory(
                    resourceMachine.getMachineHost(), Integer.parseInt(resourceMachine.getSshPort()), resourceMachine.getUsername(), resourceMachine.getPassword(), null, remoteFilePath + "PIP_" + req.getPipelineId() + crashesFilePath);

            if (!CollectionUtils.isEmpty(files)) {
                // 步骤2：批量复制文件到目标目录
                List<String> copiedFiles = SftpUploadUtil.copyRemoteFilesToLocal(
                        resourceMachine.getMachineHost(), Integer.parseInt(resourceMachine.getSshPort()), resourceMachine.getUsername(), resourceMachine.getPassword(), null, files, "/home/ops/opsFile/crashes_result/" + UUID.randomUUID() + "/");
                System.out.println("共复制 " + copiedFiles.size() + " 个文件");

                if (!CollectionUtils.isEmpty(copiedFiles)){
                    List<AflCrashesInfo> aflCrashesInfos = new ArrayList<>(0);
                    //新增数据到crashes表中
                    copiedFiles.forEach(o->{
                        AflCrashesInfo aflCrashesInfo = new AflCrashesInfo();
                        aflCrashesInfo.setFilePath(o);
                        aflCrashesInfo.setPipelineId(req.getPipelineId());
                        if (!StringUtils.isEmpty(req.getTaskId())){
                            aflCrashesInfo.setTaskId(req.getTaskId());
                        }
                        if (!StringUtils.isEmpty(req.getPipelineHistoryId())){
                            aflCrashesInfo.setPipelineHistoryId(req.getPipelineHistoryId());
                        }
                        aflCrashesInfo.setGroupIdentifier(uuid);
                        aflCrashesInfos.add(aflCrashesInfo);
                    });
                    baseMapper.insertBatch(aflCrashesInfos);
                }
                return uuid;
            }else {
                return null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"crashes入库失败");
        }
    }

    @Override
    public void updateHistoryPipelineIdByAflCrashesInfo(AflManagerReq req) {
        if (!StringUtils.isEmpty(req.getGroupIdentifier())){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"groupIdentifier不能为空");
        }
        QueryWrapper<AflCrashesInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("group_identifier", req.getGroupIdentifier());
        List<AflCrashesInfo> aflCrashesInfos = baseMapper.selectList(wrapper);
        if (!Collections.isEmpty(aflCrashesInfos)){
            aflCrashesInfos.forEach(o->{
                o.setPipelineHistoryId(req.getPipelineHistoryId());
            });
        }
        baseMapper.updateBatch(aflCrashesInfos);
    }

    @Override
    public List<AflCrashesInfoResp> findPipelineList(AflManagerReq req) {
        QueryWrapper<AflCrashesInfo> wrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(req.getId())){
            wrapper.eq("id",req.getId());
        }
        if (!ObjectUtils.isEmpty(req.getPipelineId())){
            wrapper.eq("pipeline_id",req.getPipelineId());
        }
        if (!ObjectUtils.isEmpty(req.getPipelineHistoryId())){
            wrapper.eq("pipeline_history_id",req.getPipelineHistoryId());
        }
        if (!ObjectUtils.isEmpty(req.getTaskId())){
            wrapper.eq("task_id",req.getTaskId());
        }
        if (!ObjectUtils.isEmpty(req.getGroupIdentifier())){
            wrapper.eq("group_identifier",req.getGroupIdentifier());
        }

        List<AflCrashesInfo> aflCrashesInfos = baseMapper.selectList(wrapper);
        if (ObjectUtils.isEmpty(aflCrashesInfos)){
            return new ArrayList<>(0);
        }

        return AflManagerConverter.INSTANCE.toAflCrashesInfoRespList(aflCrashesInfos);
    }
}
